@@ -318,8 +318,7 @@ var WindowListener = {
           }
         });
 
-        // Disabled for 45, as 45 has its own menuitem.
-        // this.addMenuItem();
+        this.addMenuItem();
 
         // Don't do the rest if this is for the hidden window - we don't
         // have a toolbar there.
