@@ -12,7 +12,9 @@ loop.panel = function (_, mozL10n) {
   var sharedActions = loop.shared.actions;
   var Button = sharedViews.Button;
 
-  var FTU_VERSION = 1;
+  // XXX This must be kept in sync with the number in MozLoopService.jsm.
+  // We should expose that one through MozLoopAPI and kill this constant.
+  var FTU_VERSION = 2;
 
   var GettingStartedView = React.createClass({
     displayName: "GettingStartedView",
@@ -20,18 +22,10 @@ loop.panel = function (_, mozL10n) {
     mixins: [sharedMixins.WindowCloseMixin],
 
     handleButtonClick: function () {
-      loop.requestMulti(["OpenGettingStartedTour", "getting-started"], ["SetLoopPref", "gettingStarted.latestFTUVersion", FTU_VERSION], ["SetPanelHeight"]).then(function () {
-        var event = new CustomEvent("GettingStartedSeen");
-        window.dispatchEvent(event);
-      }.bind(this));
+      loop.request("OpenGettingStartedTour");
       this.closeWindow();
     },
 
-    componentWillMount: function () {
-      // Show the full FTU panel content.
-      loop.request("SetPanelHeight", 442);
-    },
-
     render: function () {
       return React.createElement(
         "div",
@@ -43,7 +37,13 @@ loop.panel = function (_, mozL10n) {
           React.createElement(
             "div",
             { className: "fte-subheader" },
-            mozL10n.get("first_time_experience_subheading")
+            mozL10n.get("first_time_experience_subheading2")
+          ),
+          React.createElement("hr", { className: "fte-separator" }),
+          React.createElement(
+            "div",
+            { className: "fte-content" },
+            mozL10n.get("first_time_experience_content")
           ),
           React.createElement("img", { className: "fte-hello-web-share", src: "shared/img/hello-web-share.svg" })
         ),
@@ -51,7 +51,7 @@ loop.panel = function (_, mozL10n) {
           "div",
           { className: "fte-button-container" },
           React.createElement(Button, { additionalClass: "fte-get-started-button",
-            caption: mozL10n.get("first_time_experience_button_label"),
+            caption: mozL10n.get("first_time_experience_button_label2"),
             htmlId: "fte-button",
             onClick: this.handleButtonClick })
         )
@@ -291,7 +291,7 @@ loop.panel = function (_, mozL10n) {
     },
 
     openGettingStartedTour: function () {
-      loop.request("OpenGettingStartedTour", "settings-menu");
+      loop.request("OpenGettingStartedTour");
       this.closeWindow();
     },
 
@@ -467,13 +467,10 @@ loop.panel = function (_, mozL10n) {
       }));
 
       // Open url if needed.
-      loop.requestMulti(
-        ["getSelectedTabMetadata"],
-        ["GettingStartedURL", null, {}]
-      ).then(function(results) {
+      loop.requestMulti(["getSelectedTabMetadata"], ["GettingStartedURL", null, {}]).then(function (results) {
         var contextURL = this.props.room.decryptedContext.urls && this.props.room.decryptedContext.urls[0].location;
 
-        contextURL = contextURL || (results[1] + "?noopenpanel=1");
+        contextURL = contextURL || results[1] + "?noopenpanel=1";
 
         if (results[0].url !== contextURL) {
           loop.request("OpenURL", contextURL);
@@ -745,14 +742,20 @@ loop.panel = function (_, mozL10n) {
      * @returns {Object} React render
      */
     _renderLoadingRoomsView: function () {
+      /* XXX should refactor and separate "rooms" amd perhaps room-list so that
+      we arent duplicating those elements all over */
       return React.createElement(
         "div",
-        { className: "room-list" },
+        { className: "rooms" },
         this._renderNewRoomButton(),
         React.createElement(
           "div",
-          { className: "room-list-loading" },
-          React.createElement("img", { src: "shared/img/animated-spinner.svg" })
+          { className: "room-list" },
+          React.createElement(
+            "div",
+            { className: "room-list-loading" },
+            React.createElement("img", { src: "shared/img/animated-spinner.svg" })
+          )
         )
       );
     },
@@ -763,7 +766,20 @@ loop.panel = function (_, mozL10n) {
         pendingOperation: this.state.pendingCreation || this.state.pendingInitialRetrieval });
     },
 
+    _addListGradientIfNeeded: function () {
+      if (this.state.rooms.length > 5) {
+        return React.createElement("div", { className: "room-list-blur" });
+      }
+    },
+
     render: function () {
+      var roomListClasses = classNames({
+        "room-list": true,
+        // add extra space to last item so when scrolling to bottom,
+        // last item is not covered by the gradient
+        "room-list-add-space": this.state.rooms.length && this.state.rooms.length > 5
+      });
+
       if (this.state.error) {
         // XXX Better end user reporting of errors.
         console.error("RoomList error", this.state.error);
@@ -782,9 +798,9 @@ loop.panel = function (_, mozL10n) {
           null,
           mozL10n.get(this.state.openedRoom === null ? "rooms_list_recently_browsed2" : "rooms_list_currently_browsing2")
         ),
-        !this.state.rooms.length ? null : React.createElement(
+        !this.state.rooms.length ? React.createElement("div", { className: "room-list-empty" }) : React.createElement(
           "div",
-          { className: "room-list" },
+          { className: roomListClasses },
           this.state.rooms.map(function (room) {
             if (this.state.openedRoom !== null && room.roomToken !== this.state.openedRoom) {
               return null;
@@ -796,7 +812,8 @@ loop.panel = function (_, mozL10n) {
               key: room.roomToken,
               room: room });
           }, this)
-        )
+        ),
+        this._addListGradientIfNeeded()
       );
     }
   });
@@ -871,7 +888,7 @@ loop.panel = function (_, mozL10n) {
           { className: "btn btn-info stop-sharing-button",
             disabled: this.props.pendingOperation,
             onClick: this.handleStopSharingButtonClick },
-          mozL10n.get("panel_stop_sharing_tabs_button")
+          mozL10n.get("panel_disconnect_button")
         ) : React.createElement(
           "button",
           { className: "btn btn-info new-room-button",
@@ -983,26 +1000,32 @@ loop.panel = function (_, mozL10n) {
     },
 
     _onStatusChanged: function () {
-      loop.requestMulti(["GetUserProfile"], ["GetHasEncryptionKey"]).then(function (results) {
+      loop.requestMulti(["GetUserProfile"], ["GetHasEncryptionKey"], ["GetLoopPref", "gettingStarted.latestFTUVersion"]).then(function (results) {
         var profile = results[0];
         var hasEncryptionKey = results[1];
+        var prefFTUVersion = results[2];
+
+        var stateToUpdate = {};
+
+        // It's possible that this state change was slideshow related
+        // so update that if the pref has changed.
+        var prefGettingStartedSeen = prefFTUVersion >= FTU_VERSION;
+        if (prefGettingStartedSeen !== this.state.gettingStartedSeen) {
+          stateToUpdate.gettingStartedSeen = prefGettingStartedSeen;
+        }
+
         var currUid = this.state.userProfile ? this.state.userProfile.uid : null;
         var newUid = profile ? profile.uid : null;
         if (currUid === newUid) {
           // Update the state of hasEncryptionKey as this might have changed now.
-          this.setState({ hasEncryptionKey: hasEncryptionKey });
+          stateToUpdate.hasEncryptionKey = hasEncryptionKey;
         } else {
-          this.setState({ userProfile: profile });
+          stateToUpdate.userProfile = profile;
         }
-        this.updateServiceErrors();
-      }.bind(this));
-    },
 
-    _gettingStartedSeen: function () {
-      loop.request("GetLoopPref", "gettingStarted.latestFTUVersion").then(function (result) {
-        this.setState({
-          gettingStartedSeen: result >= FTU_VERSION
-        });
+        this.setState(stateToUpdate);
+
+        this.updateServiceErrors();
       }.bind(this));
     },
 
@@ -1012,12 +1035,10 @@ loop.panel = function (_, mozL10n) {
 
     componentDidMount: function () {
       loop.subscribe("LoopStatusChanged", this._onStatusChanged);
-      window.addEventListener("GettingStartedSeen", this._gettingStartedSeen);
     },
 
     componentWillUnmount: function () {
       loop.unsubscribe("LoopStatusChanged", this._onStatusChanged);
-      window.removeEventListener("GettingStartedSeen", this._gettingStartedSeen);
     },
 
     handleContextMenu: function (e) {
